/* ***********************************************************************/
/*                                                                       */
/*                       WiNRADiO G39DDC EXCELSIOR                       */
/*               C/C++ header file for G39DDC receiver API               */
/*           (C) Copyright 2015 RADIXON. All rights reserved.            */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G39DDCAPI_H__
#define __G39DDCAPI_H__

#include <stdint.h>

#pragma pack(push,1)

#define G39DDC_CLASS_ID_DEVICE_ENUMERATOR   0x1001
#define G39DDC_CLASS_ID_DEVICE              0x1002

#define G39DDC_INTERFACE_TYPE_PCIE          0
#define G39DDC_INTERFACE_TYPE_USB           1

#define G39DDC_OPEN_DEMO   ((char*)0)
#define G39DDC_OPEN_FIRST  ((char*)-1l)

#define G39DDC_FRONT_PANEL_LED_MODE_DIAG    0
#define G39DDC_FRONT_PANEL_LED_MODE_ON      1
#define G39DDC_FRONT_PANEL_LED_MODE_OFF     2

#define G39DDC_ATTENUATOR_0dB               0
#define G39DDC_ATTENUATOR_6dB               1
#define G39DDC_ATTENUATOR_12dB              2
#define G39DDC_ATTENUATOR_18dB              3
#define G39DDC_ATTENUATOR_MAX               G39DDC_ATTENUATOR_18dB

#define G39DDC_HARDWARE_OPTIONS_EXTERNAL_REFERENCE      0x00000001
#define G39DDC_HARDWARE_OPTIONS_AUDIO_OUTPUT            0x00000002
#define G39DDC_HARDWARE_OPTIONS_AUDIO_OUTPUT_WIDE       0x00000004
#define G39DDC_HARDWARE_OPTIONS_FCC                     0x80000000

#define G39DDC_DEVICE_STATE_TEMP_ERROR                  0x00000001
#define G39DDC_DEVICE_STATE_FAN_ERROR                   0x00000002
#define G39DDC_DEVICE_STATE_VCC_ERROR                   0x00000004
#define G39DDC_DEVICE_STATE_COMM_ERROR                  0x00000008
#define G39DDC_DEVICE_STATE_VCC_LOW_ERROR               0x00000010
#define G39DDC_DEVICE_STATE_VCC_HIGH_ERROR              0x00000020
#define G39DDC_DEVICE_STATE_FAN_OVERLOAD_ERROR          0x00000040
#define G39DDC_DEVICE_STATE_FAN_NOTWORK_ERROR           0x00000080
#define G39DDC_DEVICE_STATE_SUSPENDED                   0x80000000

#define G39DDC_MODE_AM                                  0x00
#define G39DDC_MODE_AMS                                 0x01
#define G39DDC_MODE_LSB                                 0x02
#define G39DDC_MODE_USB                                 0x03
#define G39DDC_MODE_DSB                                 0x04
#define G39DDC_MODE_ISB                                 0x05
#define G39DDC_MODE_CW                                  0x06
#define G39DDC_MODE_FM                                  0x07
#define G39DDC_MODE_FMW                                 0x08
#define G39DDC_MODE_DRM                                 0x09 /*not supported*/
#define G39DDC_MODE_COUNT                               (G39DDC_MODE_FMW+1)

#define G39DDC_SIDE_BAND_LOWER         0x01
#define G39DDC_SIDE_BAND_UPPER         0x02
#define G39DDC_SIDE_BAND_BOTH          (G39DDC_SIDE_BAND_LOWER|G39DDC_SIDE_BAND_UPPER)

#define G39DDC_DEMODULATOR_PARAM_AMS_SIDE_BAND          0x00000001
#define G39DDC_DEMODULATOR_PARAM_ISB_SIDE_BAND          0x00000002
#define G39DDC_DEMODULATOR_PARAM_DSB_SIDE_BAND          0x00000003
#define G39DDC_DEMODULATOR_PARAM_FMW_STEREO             0x00000004
#define G39DDC_DEMODULATOR_PARAM_CW_FREQUENCY           0x00000005
#define G39DDC_DEMODULATOR_PARAM_FMW_DEEMPHASIS         0x00000006
#define G39DDC_DEMODULATOR_PARAM_DRM_AUDIO_SERVICE      0x00000007
#define G39DDC_DEMODULATOR_PARAM_DRM_MULTIMEDIA_SERVICE 0x00000008
#define G39DDC_DEMODULATOR_PARAM_AMS_CAPTURE_RANGE      0x00000009

#define G39DDC_DEMODULATOR_STATE_AMS_LOCK               0x00000001
#define G39DDC_DEMODULATOR_STATE_AMS_FREQUENCY          0x00000002
#define G39DDC_DEMODULATOR_STATE_DSB_LOCK               0x00000003
#define G39DDC_DEMODULATOR_STATE_DSB_FREQUENCY          0x00000004
#define G39DDC_DEMODULATOR_STATE_AM_DEPTH               0x00000005
#define G39DDC_DEMODULATOR_STATE_TUNE_ERROR             0x00000006
#define G39DDC_DEMODULATOR_STATE_DRM_STATUS             0x00000007
#define G39DDC_DEMODULATOR_STATE_FMW_STEREO             0x00000008
#define G39DDC_DEMODULATOR_STATE_FM_DEVIATION           0x00000009

#define G39DDC_FLUSH_BUFFERS_IF                         0x0001
#define G39DDC_FLUSH_BUFFERS_DDC1                       0x0002
#define G39DDC_FLUSH_BUFFERS_DDC2                       0x0004
#define G39DDC_FLUSH_BUFFERS_AUDIO                      0x0008

#define G39DDC_DDC1_MODE_DEFAULT                        0
#define G39DDC_DDC1_MODE_WIDEBAND                       1

typedef int (*G39DDCAPI_CREATE_INSTANCE)(uint32_t Code,void **Intf);

typedef struct
{
    char        DevicePath[64];
    uint8_t     InterfaceType;
    char        SerialNumber[9];
    uint16_t    HWVersion;
    uint16_t    FWVersion;
    uint8_t     EEPROMVersion;
    uint32_t    ChannelCount;
    uint64_t    FrontEndMinFrequency;
    uint64_t    FrontEndMaxFrequency;
    uint64_t    FrontEndMaxHFFrequency;
    uint32_t    FrontEndFrequencyStep;
    uint32_t    FrontEndWindowWidth;
    uint32_t    HardwareOptions;
} G39DDC_DEVICE_INFO;

typedef struct
{
    uint32_t    SampleRate;
    uint32_t    Bandwidth;
    uint32_t    BitsPerSample;
} G39DDC_DDC_INFO;

typedef struct
{
    uint32_t    Tune;   //Hz
    uint32_t    Lock;   //Hz
} G39DDC_AMS_CAPTURE_RANGE;

#ifdef __cplusplus

#define G39DDC_DECLARE_INTERFACE(__name__) struct __name__

#define G39DDC_THIS

#define G39DDC_METHOD(__type__,__method__,__params__) virtual __type__ __method__ __params__=0

#define G39DDC_METHOD_VOID(__type__,__method__) virtual __type__ __method__(void)=0

#else //__cplusplus

#define G39DDC_THIS struct G39DDC_INTERFACE *This,

#define G39DDC_DECLARE_INTERFACE(__name__) typedef struct __name__ { const struct __name__##Vtbl *lpVtbl;} __name__;\
    typedef const struct __name__##Vtbl __name__##Vtbl;struct __name__##Vtbl

#define G39DDC_METHOD(__type__,__method__,__params__) __type__ (*__method__)__params__
#define G39DDC_METHOD_VOID(__type__,__method__) __type__ (*__method__)(struct G39DDC_INTERFACE *This)


#endif //__cplusplus


#undef G39DDC_INTERFACE
#define G39DDC_INTERFACE IG39DDCDeviceEnumerator

G39DDC_DECLARE_INTERFACE(IG39DDCDeviceEnumerator)
{
    G39DDC_METHOD_VOID(uint32_t,AddRef);
    G39DDC_METHOD_VOID(uint32_t,Release);

    G39DDC_METHOD_VOID(int,Enumerate);
    G39DDC_METHOD_VOID(uint32_t,GetCount);
    G39DDC_METHOD(int,GetDeviceInfo,(G39DDC_THIS uint32_t DeviceIndex,G39DDC_DEVICE_INFO *DeviceInfo,uint32_t BufferLength));
};


#undef G39DDC_INTERFACE
#define G39DDC_INTERFACE IG39DDCDeviceCallback

struct IG39DDCDevice;

G39DDC_DECLARE_INTERFACE(IG39DDCDeviceCallback)
{
    G39DDC_METHOD(void,G39DDC_DDC1StreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample));
    G39DDC_METHOD(int,G39DDC_DDC1PlaybackStreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample));
    G39DDC_METHOD(void,G39DDC_DDC2StreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples));
    G39DDC_METHOD(void,G39DDC_DDC2PreprocessedStreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS));
    G39DDC_METHOD(void,G39DDC_IFCallback,(G39DDC_THIS struct IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate));
    G39DDC_METHOD(int,G39DDC_SweepingCallback,(G39DDC_THIS struct IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint64_t FrontEndFrequency));
    G39DDC_METHOD(void,G39DDC_AudioStreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples));
    G39DDC_METHOD(int,G39DDC_AudioPlaybackStreamCallback,(G39DDC_THIS struct IG39DDCDevice *Device,uint32_t Channel,float *Buffer,uint32_t NumberOfSamples));
};

#undef G39DDC_INTERFACE
#define G39DDC_INTERFACE IG39DDCDevice

G39DDC_DECLARE_INTERFACE(IG39DDCDevice)
{
    G39DDC_METHOD_VOID(uint32_t,AddRef);
    G39DDC_METHOD_VOID(uint32_t,Release);

    G39DDC_METHOD(int,Open,(G39DDC_THIS const char *DevicePath));
    G39DDC_METHOD_VOID(void,Close);

    G39DDC_METHOD_VOID(int,IsConnected);

    G39DDC_METHOD(void,SetCallback,(G39DDC_THIS IG39DDCDeviceCallback *Callback));
    G39DDC_METHOD_VOID(IG39DDCDeviceCallback*,GetCallback);

    G39DDC_METHOD(int,GetDeviceInfo,(G39DDC_THIS G39DDC_DEVICE_INFO *Info,uint32_t BufferLength));

    G39DDC_METHOD(int,SetLED,(G39DDC_THIS uint32_t LEDMode));
    G39DDC_METHOD(int,GetLED,(G39DDC_THIS uint32_t *LEDMode));

    G39DDC_METHOD(int,SetPower,(G39DDC_THIS int Power));
    G39DDC_METHOD(int,GetPower,(G39DDC_THIS int *Power));

    G39DDC_METHOD(int,SetExternalReference,(G39DDC_THIS int Enabled));
    G39DDC_METHOD(int,GetExternalReference,(G39DDC_THIS int *Enabled));

    G39DDC_METHOD(int,SetFrontEndFrequency,(G39DDC_THIS uint64_t Frequency));
    G39DDC_METHOD(int,GetFrontEndFrequency,(G39DDC_THIS uint64_t *Frequency));

    G39DDC_METHOD(int,SetAttenuator,(G39DDC_THIS uint32_t Attenuator));
    G39DDC_METHOD(int,GetAttenuator,(G39DDC_THIS uint32_t *Attenuator));
    G39DDC_METHOD(int,SetPreamplifier,(G39DDC_THIS int Preamp));
    G39DDC_METHOD(int,GetPreamplifier,(G39DDC_THIS int *Preamp));

    G39DDC_METHOD(int,SetDithering,(G39DDC_THIS int Dithering));
    G39DDC_METHOD(int,GetDithering,(G39DDC_THIS int *Dithering));

    G39DDC_METHOD(int,SetADCNoiseBlanker,(G39DDC_THIS int Enabled));
    G39DDC_METHOD(int,GetADCNoiseBlanker,(G39DDC_THIS int *Enabled));
    G39DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G39DDC_THIS uint16_t Threshold));
    G39DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G39DDC_THIS uint16_t *Threshold));

    G39DDC_METHOD(int,SetFrequency,(G39DDC_THIS uint32_t Channel,uint64_t Frequency));
    G39DDC_METHOD(int,GetFrequency,(G39DDC_THIS uint32_t Channel,uint64_t *Frequency));

    G39DDC_METHOD(int,StartIF,(G39DDC_THIS uint16_t Period));
    G39DDC_METHOD_VOID(int,StopIF);

    G39DDC_METHOD(int,GetDDCInfo,(G39DDC_THIS uint32_t DDCIndex,G39DDC_DDC_INFO *Info));

    G39DDC_METHOD(int,GetDDC1Count,(G39DDC_THIS uint32_t Channel,uint32_t *Count));
    G39DDC_METHOD(int,SetDDC1,(G39DDC_THIS uint32_t Channel,uint32_t DDCIndex));
    G39DDC_METHOD(int,GetDDC1,(G39DDC_THIS uint32_t Channel,uint32_t *DDCIndex));
    G39DDC_METHOD(int,StartDDC1,(G39DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G39DDC_METHOD(int,StopDDC1,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,SetDDC1Frequency,(G39DDC_THIS uint32_t Channel,int32_t Frequency));
    G39DDC_METHOD(int,GetDDC1Frequency,(G39DDC_THIS uint32_t Channel,int32_t *Frequency));
    G39DDC_METHOD(int,StartDDC1Playback,(G39DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer,uint32_t BitsPerSample));
    G39DDC_METHOD(int,PauseDDC1Playback,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,ResumeDDC1Playback,(G39DDC_THIS uint32_t Channel));

    G39DDC_METHOD(int,GetDDC2Count,(G39DDC_THIS uint32_t Channel,uint32_t *Count));
    G39DDC_METHOD(int,SetDDC2,(G39DDC_THIS uint32_t Channel,uint32_t DDCIndex));
    G39DDC_METHOD(int,GetDDC2,(G39DDC_THIS uint32_t Channel,uint32_t *DDCIndex));
    G39DDC_METHOD(int,StartDDC2,(G39DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G39DDC_METHOD(int,StopDDC2,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,SetDDC2Frequency,(G39DDC_THIS uint32_t Channel,int32_t Frequency));
    G39DDC_METHOD(int,GetDDC2Frequency,(G39DDC_THIS uint32_t Channel,int32_t *Frequency));

    G39DDC_METHOD(int,SetAGC,(G39DDC_THIS uint32_t Channel,int Enabled));
    G39DDC_METHOD(int,GetAGC,(G39DDC_THIS uint32_t Channel,int *Enabled));
    G39DDC_METHOD(int,SetAGCParams,(G39DDC_THIS uint32_t Channel,double AttackTime,double DecayTime,double ReferenceLevel));
    G39DDC_METHOD(int,GetAGCParams,(G39DDC_THIS uint32_t Channel,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G39DDC_METHOD(int,SetMaxAGCGain,(G39DDC_THIS uint32_t Channel,double MaxGain));
    G39DDC_METHOD(int,GetMaxAGCGain,(G39DDC_THIS uint32_t Channel,double *MaxGain));
    G39DDC_METHOD(int,SetGain,(G39DDC_THIS uint32_t Channel,double Gain));
    G39DDC_METHOD(int,GetGain,(G39DDC_THIS uint32_t Channel,double *Gain));
    G39DDC_METHOD(int,GetCurrentGain,(G39DDC_THIS uint32_t Channel,double *Gain));

    G39DDC_METHOD(int,SetNotchFilter,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,int Enabled));
    G39DDC_METHOD(int,GetNotchFilter,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,int *Enabled));
    G39DDC_METHOD(int,SetNotchFilterFrequency,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,int32_t Frequency));
    G39DDC_METHOD(int,GetNotchFilterFrequency,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,int32_t *Frequency));
    G39DDC_METHOD(int,SetNotchFilterBandwidth,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t Bandwidth));
    G39DDC_METHOD(int,GetNotchFilterBandwidth,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t *Bandwidth));
    G39DDC_METHOD(int,SetNotchFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t Length));
    G39DDC_METHOD(int,GetNotchFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t FilterIndex,uint32_t *Length));

    G39DDC_METHOD(int,SetDemodulatorFrequency,(G39DDC_THIS uint32_t Channel,int32_t Frequency));
    G39DDC_METHOD(int,GetDemodulatorFrequency,(G39DDC_THIS uint32_t Channel,int32_t *Frequency));
    G39DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G39DDC_THIS uint32_t Channel,uint32_t Bandwidth));
    G39DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G39DDC_THIS uint32_t Channel,uint32_t *Bandwidth));
    G39DDC_METHOD(int,SetDemodulatorFilterShift,(G39DDC_THIS uint32_t Channel,int32_t Shift));
    G39DDC_METHOD(int,GetDemodulatorFilterShift,(G39DDC_THIS uint32_t Channel,int32_t *Shift));
    G39DDC_METHOD(int,SetDemodulatorFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t Length));
    G39DDC_METHOD(int,GetDemodulatorFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t *Length));
    G39DDC_METHOD(int,SetDemodulatorMode,(G39DDC_THIS uint32_t Channel,uint32_t Mode));
    G39DDC_METHOD(int,GetDemodulatorMode,(G39DDC_THIS uint32_t Channel,uint32_t *Mode));
    G39DDC_METHOD(int,SetDemodulatorParam,(G39DDC_THIS uint32_t Channel,uint32_t Code,const void *Buffer,uint32_t BufferSize));
    G39DDC_METHOD(int,GetDemodulatorParam,(G39DDC_THIS uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize));
    G39DDC_METHOD(int,GetDemodulatorState,(G39DDC_THIS uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize));

    G39DDC_METHOD(int,GetAudioSampleRateCount,(G39DDC_THIS uint32_t *Count));
    G39DDC_METHOD(int,GetAudioSampleRate,(G39DDC_THIS uint32_t SampleRateIndex,uint32_t *SampleRate));
    G39DDC_METHOD(int,StartAudio,(G39DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G39DDC_METHOD(int,StopAudio,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,SetAudio,(G39DDC_THIS uint32_t Channel,uint32_t SampleRateIndex));
    G39DDC_METHOD(int,GetAudio,(G39DDC_THIS uint32_t Channel,uint32_t *SampleRateIndex));
    G39DDC_METHOD(int,StartAudioPlayback,(G39DDC_THIS uint32_t Channel,uint32_t SamplesPerBuffer));
    G39DDC_METHOD(int,PauseAudioPlayback,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,ResumeAudioPlayback,(G39DDC_THIS uint32_t Channel));
    G39DDC_METHOD(int,SetAudioFilter,(G39DDC_THIS uint32_t Channel,int Enabled));
    G39DDC_METHOD(int,GetAudioFilter,(G39DDC_THIS uint32_t Channel,int *Enabled));
    G39DDC_METHOD(int,SetAudioFilterParams,(G39DDC_THIS uint32_t Channel,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G39DDC_METHOD(int,GetAudioFilterParams,(G39DDC_THIS uint32_t Channel,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G39DDC_METHOD(int,SetAudioFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t Length));
    G39DDC_METHOD(int,GetAudioFilterLength,(G39DDC_THIS uint32_t Channel,uint32_t *Length));
    G39DDC_METHOD(int,SetAudioGain,(G39DDC_THIS uint32_t Channel,double Gain));
    G39DDC_METHOD(int,GetAudioGain,(G39DDC_THIS uint32_t Channel,double *Gain));
    G39DDC_METHOD(int,SetVolume,(G39DDC_THIS uint32_t Channel,uint8_t Volume));
    G39DDC_METHOD(int,GetVolume,(G39DDC_THIS uint32_t Channel,uint8_t *Volume));
    G39DDC_METHOD(int,SetMute,(G39DDC_THIS uint32_t Channel,int Mute));
    G39DDC_METHOD(int,GetMute,(G39DDC_THIS uint32_t Channel,int *Mute));

    G39DDC_METHOD(int,StartSweeping,(G39DDC_THIS uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint16_t SettlingTime,int Forward));
    G39DDC_METHOD_VOID(int,StopSweeping);

    G39DDC_METHOD(int,GetSpectrumCompensation,(G39DDC_THIS uint64_t FrontEndFrequency,int32_t Shift,uint32_t Bandwidth,float *Buffer,uint32_t Count));
    G39DDC_METHOD(int,GetIFSpectrumCompensation,(G39DDC_THIS uint64_t FrontEndFrequency,float *Buffer,uint32_t Count));

    G39DDC_METHOD(int,SetDAC,(G39DDC_THIS uint32_t DAC));
    G39DDC_METHOD(int,GetDAC,(G39DDC_THIS uint32_t *DAC));

    G39DDC_METHOD(int,FlushBuffers,(G39DDC_THIS uint32_t Channel,uint32_t Flags));

    G39DDC_METHOD(int,GetTemperature,(G39DDC_THIS uint32_t *Temperature));

    G39DDC_METHOD(int,SetInverted,(G39DDC_THIS int Inverted));
    G39DDC_METHOD(int,GetInverted,(G39DDC_THIS int *Inverted));

    G39DDC_METHOD(int,SetDDC2NoiseBlanker,(G39DDC_THIS uint32_t Channel,int Enabled));
    G39DDC_METHOD(int,GetDDC2NoiseBlanker,(G39DDC_THIS uint32_t Channel,int *Enabled));
    G39DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G39DDC_THIS uint32_t Channel,double Threshold));
    G39DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G39DDC_THIS uint32_t Channel,double *Threshold));
    G39DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G39DDC_THIS uint32_t Channel,double *Value));

    G39DDC_METHOD(int,GetDeviceState,(G39DDC_THIS uint32_t *State));

    G39DDC_METHOD(int,GetSignalLevel,(G39DDC_THIS uint32_t Channel,float *Peak,float *RMS));

    G39DDC_METHOD(int,StartSweepingEx,(G39DDC_THIS uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint16_t SettlingTime,int Forward,uint32_t Repeat,int LargeIfBuffer));

    G39DDC_METHOD(int,SetDDC1Mode,(G39DDC_THIS uint32_t DDC1Mode));
    G39DDC_METHOD(int,GetDDC1Mode,(G39DDC_THIS uint32_t *DDC1Mode));
};


typedef void (*G39DDC_DDC1_STREAM_CALLBACK)(uint32_t Channel,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample,uintptr_t UserData);
typedef int (*G39DDC_DDC1_PLAYBACK_STREAM_CALLBACK)(uint32_t Channel,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*G39DDC_DDC2_STREAM_CALLBACK)(uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);
typedef void (*G39DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS,uintptr_t UserData);
typedef void (*G39DDC_IF_CALLBACK)(const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate,uintptr_t UserData);
typedef int (*G39DDC_SWEEPING_CALLBACK)(const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint64_t FrontEndFrequency,uintptr_t UserData);
typedef void (*G39DDC_AUDIO_STREAM_CALLBACK)(uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);
typedef int (*G39DDC_AUDIO_PLAYBACK_STREAM_CALLBACK)(uint32_t Channel,float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData);

typedef struct
{
    G39DDC_IF_CALLBACK                          IFCallback;
    G39DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    G39DDC_DDC1_PLAYBACK_STREAM_CALLBACK        DDC1PlaybackStreamCallback;
    G39DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    G39DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    G39DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
    G39DDC_AUDIO_PLAYBACK_STREAM_CALLBACK       AudioPlaybackStreamCallback;
    G39DDC_SWEEPING_CALLBACK                    SweepingCallback;
} G39DDC_CALLBACKS;

typedef int (*G39DDC_GET_DEVICE_LIST)(G39DDC_DEVICE_INFO *DeviceList,uint32_t BufferSize,uint32_t *Count,uint32_t *DeviceListItemSize);
typedef int (*G39DDC_OPEN_DEVICE)(const char *SerialNumber);
typedef int (*G39DDC_CLOSE_DEVICE)(int32_t Handle);
typedef int (*G39DDC_IS_DEVICE_CONNECTED)(int32_t Handle);
typedef int (*G39DDC_GET_DEVICE_INFO)(int32_t Handle,G39DDC_DEVICE_INFO *Info,uint32_t BufferLength);
typedef int (*G39DDC_SET_LED)(int32_t Handle,uint32_t LEDMode);
typedef int (*G39DDC_GET_LED)(int32_t Handle,uint32_t *LEDMode);
typedef int (*G39DDC_SET_POWER)(int32_t Handle,int Power);
typedef int (*G39DDC_GET_POWER)(int32_t Handle,int *Power);
typedef int (*G39DDC_SET_EXTERNAL_REFERENCE)(int32_t Handle,int Enabled);
typedef int (*G39DDC_GET_EXTERNAL_REFERENCE)(int32_t Handle,int *Enabled);
typedef int (*G39DDC_SET_FRONT_END_FREQUENCY)(int32_t Handle,uint64_t Frequency);
typedef int (*G39DDC_GET_FRONT_END_FREQUENCY)(int32_t Handle,uint64_t *Frequency);
typedef int (*G39DDC_SET_ATTENUATOR)(int32_t Handle,uint32_t Attenuator);
typedef int (*G39DDC_GET_ATTENUATOR)(int32_t Handle,uint32_t *Attenuator);
typedef int (*G39DDC_SET_PREAMPLIFIER)(int32_t Handle,int Preamp);
typedef int (*G39DDC_GET_PREAMPLIFIER)(int32_t Handle,int *Preamp);
typedef int (*G39DDC_SET_DITHERING)(int32_t Handle,int Dithering);
typedef int (*G39DDC_GET_DITHERING)(int32_t Handle,int *Dithering);
typedef int (*G39DDC_SET_ADC_NOISE_BLANKER)(int32_t Handle,int Enabled);
typedef int (*G39DDC_GET_ADC_NOISE_BLANKER)(int32_t Handle,int *Enabled);
typedef int (*G39DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint16_t Threshold);
typedef int (*G39DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint16_t *Threshold);
typedef int (*G39DDC_SET_FREQUENCY)(int32_t Handle,uint32_t Channel,uint64_t Frequency);
typedef int (*G39DDC_GET_FREQUENCY)(int32_t Handle,uint32_t Channel,uint64_t *Frequency);
typedef int (*G39DDC_START_IF)(int32_t Handle,uint16_t Period);
typedef int (*G39DDC_STOP_IF)(int32_t Handle);
typedef int (*G39DDC_GET_DDC_INFO)(uint32_t DDCIndex,G39DDC_DDC_INFO *Info);
typedef int (*G39DDC_GET_DDC1_COUNT)(int32_t Handle,uint32_t Channel,uint32_t *Count);
typedef int (*G39DDC_SET_DDC1)(int32_t Handle,uint32_t Channel,uint32_t DDCIndex);
typedef int (*G39DDC_GET_DDC1)(int32_t Handle,uint32_t Channel,uint32_t *DDCIndex);
typedef int (*G39DDC_START_DDC1)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G39DDC_STOP_DDC1)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_SET_DDC1_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t Frequency);
typedef int (*G39DDC_GET_DDC1_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t *Frequency);
typedef int (*G39DDC_START_DDC1_PLAYBACK)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer,uint32_t BitsPerSample);
typedef int (*G39DDC_PAUSE_DDC1_PLAYBACK)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_RESUME_DDC1_PLAYBACK)(int32_t Handle,uint32_t Channel);

typedef int (*G39DDC_GET_DDC2_COUNT)(int32_t Handle,uint32_t Channel,uint32_t *Count);
typedef int (*G39DDC_SET_DDC2)(int32_t Handle,uint32_t Channel,uint32_t DDCIndex);
typedef int (*G39DDC_GET_DDC2)(int32_t Handle,uint32_t Channel,uint32_t *DDCIndex);
typedef int (*G39DDC_START_DDC2)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G39DDC_STOP_DDC2)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_SET_DDC2_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t Frequency);
typedef int (*G39DDC_GET_DDC2_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t *Frequency);

typedef int (*G39DDC_SET_AGC)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G39DDC_GET_AGC)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G39DDC_SET_AGC_PARAMS)(int32_t Handle,uint32_t Channel,double AttackTime,double DecayTime,double ReferenceLevel);
typedef int (*G39DDC_GET_AGC_PARAMS)(int32_t Handle,uint32_t Channel,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef int (*G39DDC_SET_MAX_AGC_GAIN)(int32_t Handle,uint32_t Channel,double MaxGain);
typedef int (*G39DDC_GET_MAX_AGC_GAIN)(int32_t Handle,uint32_t Channel,double *MaxGain);
typedef int (*G39DDC_SET_GAIN)(int32_t Handle,uint32_t Channel,double Gain);
typedef int (*G39DDC_GET_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);
typedef int (G39DDC_GET_CURRENT_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);

typedef int (*G39DDC_SET_NOTCH_FILTER)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int Enabled);
typedef int (*G39DDC_GET_NOTCH_FILTER)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int *Enabled);
typedef int (*G39DDC_SET_NOTCH_FILTER_FREEQUENCY)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int32_t Frequency);
typedef int (*G39DDC_GET_NOTCH_FILTER_FREEQUENCY)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,int32_t *Frequency);
typedef int (*G39DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t Bandwidth);
typedef int (*G39DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t *Bandwidth);
typedef int (*G39DDC_SET_NOTCH_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t Length);
typedef int (*G39DDC_GET_NOTCH_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t FilterIndex,uint32_t *Length);

typedef int (*G39DDC_SET_DEMODULATOR_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t Frequency);
typedef int (*G39DDC_GET_DEMODULATOR_FREQUENCY)(int32_t Handle,uint32_t Channel,int32_t *Frequency);
typedef int (*G39DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t Bandwidth);
typedef int (*G39DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t Handle,uint32_t Channel,uint32_t *Bandwidth);
typedef int (*G39DDC_SET_DEMODULATOR_FILTER_SHIFT)(int32_t Handle,uint32_t Channel,int32_t Shift);
typedef int (*G39DDC_GET_DEMODULATOR_FILTER_SHIFT)(int32_t Handle,uint32_t Channel,int32_t *Shift);
typedef int (*G39DDC_SET_DEMODULATOR_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t Length);
typedef int (*G39DDC_GET_DEMODULATOR_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t *Length);
typedef int (*G39DDC_SET_DEMODULATOR_MODE)(int32_t Handle,uint32_t Channel,uint32_t Mode);
typedef int (*G39DDC_GET_DEMODULATOR_MODE)(int32_t Handle,uint32_t Channel,uint32_t *Mode);
typedef int (*G39DDC_SET_DEMODULATOR_PARAM)(int32_t Handle,uint32_t Channel,uint32_t Code,const void *Buffer,uint32_t BufferSize);
typedef int (*G39DDC_GET_DEMODULATOR_PARAM)(int32_t Handle,uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G39DDC_GET_DEMODULATOR_STATE)(int32_t Handle,uint32_t Channel,uint32_t Code,void *Buffer,uint32_t BufferSize);

typedef int (*G39DDC_GET_AUDIO_SAMPLE_RATE_COUNT)(int32_t Handle,uint32_t *Count);
typedef int (*G39DDC_GET_AUDIO_SAMPLE_RATE)(int32_t Handle,uint32_t SampleRateIndex,uint32_t *SampleRate);
typedef int (*G39DDC_START_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G39DDC_STOP_AUDIO)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_SET_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t SampleRateIndex);
typedef int (*G39DDC_GET_AUDIO)(int32_t Handle,uint32_t Channel,uint32_t *SampleRateIndex);
typedef int (*G39DDC_START_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel,uint32_t SamplesPerBuffer);
typedef int (*G39DDC_PAUSE_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_RESUME_AUDIO_PLAYBACK)(int32_t Handle,uint32_t Channel);
typedef int (*G39DDC_SET_AUDIO_FILTER)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G39DDC_GET_AUDIO_FILTER)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G39DDC_SET_AUDIO_FILTER_PARAMS)(int32_t Handle,uint32_t Channel,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*G39DDC_GET_AUDIO_FILTER_PARAMS)(int32_t Handle,uint32_t Channel,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*G39DDC_SET_AUDIO_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t Length);
typedef int (*G39DDC_GET_AUDIO_FILTER_LENGTH)(int32_t Handle,uint32_t Channel,uint32_t *Length);
typedef int (*G39DDC_SET_AUDIO_GAIN)(int32_t Handle,uint32_t Channel,double Gain);
typedef int (*G39DDC_GET_AUDIO_GAIN)(int32_t Handle,uint32_t Channel,double *Gain);
typedef int (*G39DDC_SET_VOLUME)(int32_t Handle,uint32_t Channel,uint8_t Volume);
typedef int (*G39DDC_GET_VOLUME)(int32_t Handle,uint32_t Channel,uint8_t *Volume);
typedef int (*G39DDC_SET_MUTE)(int32_t Handle,uint32_t Channel,int Mute);
typedef int (*G39DDC_GET_MUTE)(int32_t Handle,uint32_t Channel,int *Mute);

typedef int (*G39DDC_START_SWEEPING)(int32_t Handle,uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint16_t SettlingTime,int Forward);
typedef int (*G39DDC_START_SWEEPING_EX)(int32_t Handle,uint64_t BeginFrequency,uint64_t EndFrequency,uint64_t StartFrequency,uint16_t SettlingTime,int Forward,uint32_t Repeat,int LargeIfBuffer);
typedef int (*G39DDC_STOP_SWEEPING)(int32_t Handle);

typedef int (*G39DDC_GET_SPECTRUM_COMPENSATION)(int32_t Handle,uint64_t FrontEndFrequency,int32_t Shift,uint32_t Bandwidth,float *Buffer,uint32_t Count);
typedef int (*G39DDC_GET_IF_SPECTRUM_COMPENSATION)(int32_t Handle,uint64_t FrontEndFrequency,float *Buffer,uint32_t Count);

typedef int (*G39DDC_SET_DAC)(int32_t Handle,uint32_t DAC);
typedef int (*G39DDC_GET_DAC)(int32_t Handle,uint32_t *DAC);

typedef int (*G39DDC_FLUSH_BUFFERS)(int32_t Handle,uint32_t Channel,uint32_t Flags);
typedef int (*G39DDC_GET_TEMPERATURE)(int32_t Handle,uint32_t *Temperature);

typedef int (*G39DDC_SET_INVERTED)(int32_t Handle,int Inverted);
typedef int (*G39DDC_GET_INVERTED)(int32_t Handle,int *Inverted);

typedef int (*G39DDC_SET_DDC2_NOISE_BLANKER)(int32_t Handle,uint32_t Channel,int Enabled);
typedef int (*G39DDC_GET_DDC2_NOISE_BLANKER)(int32_t Handle,uint32_t Channel,int *Enabled);
typedef int (*G39DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint32_t Channel,double Threshold);
typedef int (*G39DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t Handle,uint32_t Channel,double *Threshold);
typedef int (*G39DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t Handle,uint32_t Channel,double *Value);

typedef int (*G39DDC_GET_DEVICE_STATE)(int32_t Handle,uint32_t *State);

typedef int (*G39DDC_GET_SIGNAL_LEVEL)(int32_t Handle,uint32_t Channel,float *Peak,float *RMS);

typedef int (*G39DDC_SET_CALLBACKS)(int32_t Handle,const G39DDC_CALLBACKS *Callbacks,uintptr_t UserData);

typedef int (*G39DDC_SET_DDC1_MODE)(int32_t Handle,uint32_t DDC1Mode);
typedef int (*G39DDC_GET_DDC1_MODE)(int32_t Handle,uint32_t *DDC1Mode);

#pragma pack(pop)

#endif // __G39DDCAPI_H__
